#version 120
#extension GL_ARB_shader_texture_lod : enable

#define Godrays
#define GodraySamples 8 //[4 8 16 32]

#define Saturation 1.0 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define Contrast 1.0 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]

//#define Vignette


uniform sampler2D colortex0;

uniform vec3 sunPosition;

uniform mat4 gbufferProjection;

uniform int worldTime;

uniform float frameTimeCounter;

uniform sampler2D normals;
uniform sampler2D depthtex0;
uniform sampler2D depthtex1;
uniform sampler2D depthtex2;
uniform sampler2D gaux1;

varying vec4 texcoord;

float timefract = worldTime;

#include "/lib/dither.glsl"

//Get the time of the day
float TimeSunrise  = ((clamp(timefract, 23000.0, 24000.0) - 23000.0) / 1000.0) + (1.0 - (clamp(timefract, 0.0, 4000.0)/4000.0));
float TimeNoon     = ((clamp(timefract, 0.0, 4000.0)) / 4000.0) - ((clamp(timefract, 8000.0, 12000.0) - 8000.0) / 4000.0);
float TimeSunset   = ((clamp(timefract, 8000.0, 12000.0) - 8000.0) / 4000.0) - ((clamp(timefract, 12000.0, 12750.0) - 12000.0) / 750.0);
float TimeMidnight = ((clamp(timefract, 12000.0, 12750.0) - 12000.0) / 750.0) - ((clamp(timefract, 23000.0, 24000.0) - 23000.0) / 1000.0);

vec3 colorGrading(in vec3 col) {
    vec3 blueCol = vec3(0.7,0.7,0.4);
    vec3 redCol = vec3(0.6, 0.8, 0.28);

    vec3 regular = vec3(0.8);

    vec3 purpleCol = mix(regular, blueCol, redCol);

    vec3 mixBlueCol = mix(blueCol, col, regular);
    vec3 mixRedCol = mix(purpleCol, col, regular);

    vec3 mixDailyCol = vec3(mixRedCol*TimeSunrise + mixBlueCol*TimeNoon + mixRedCol*TimeSunset + mixBlueCol*TimeMidnight);

    return mixDailyCol;

}

vec3 getSSGR(vec2 coord) {
  vec3 ssgr = vec3(0.0);
  float sample = GodraySamples;
  vec4 projectedSun = gbufferProjection*vec4(sunPosition, 1.0);
  vec2 screenSun = ((projectedSun.xyz/projectedSun.w)*0.5+0.5).xy;
  vec2 sunDirection = coord-screenSun;

  for (int i = 0; i < sample; ++i) {
    vec2 steppedPos = sunDirection * ((i+bayer16(gl_FragCoord.xy)) / sample) + screenSun;
    float depth = texture2D(depthtex0, steppedPos).r;
    if (depth == 1.0)
      ssgr += texture2DLod(colortex0, steppedPos, 1.0).rgb*vec3(0.9, 0.57, 0.1)/2.0;
  }
  return ssgr / sample;
}

void SoftVignette(inout vec3 color) {
    float dist = distance(texcoord.st, vec2(0.5)) * 2.0;
    dist /= 1.9142f;

    dist = pow(dist, 0.8f);

    color.rgb *= (1.0f - dist) / 1.05;

}

vec3 getSaturate(vec3 color){
    float mixrgb = (color.r + color.g + color.b) / 3.5;
    float weight = Saturation + (1.0 - pow(1.0 - 1.0 * mixrgb, 2.0)) * 0.08;

    return mix(vec3(mixrgb), color, weight);
}

vec3 getContrast(vec3 color) {
    vec3 l = log2(color+1e-4);
    l = (l +1.0)*Contrast-1.0;
    return exp2(l)-1e-4;
}

void main() {
    vec3 color = texture2D(colortex0, texcoord.st).rgb;

    #ifdef Vignette
    SoftVignette(color);
    #endif

    #ifdef Godrays
    color += getSSGR(texcoord.st);
    #endif

    color = getSaturate(color);
    color = getContrast(color);

/*DRAWBUFFERS:0*/
    gl_FragData[0] = vec4(color, 1.0);
}