#version 120

uniform sampler2D noisetex;
uniform sampler2D tex;

varying vec4 texcoord;
varying vec4 color;

varying vec3 viewVector;
varying vec3 worldPos;
varying vec3 binormal;
varying vec3 normal;
varying vec3 tangent;
varying mat3 tbnMatrix;

varying float id;

uniform int frameTimeCounter;

uniform float sunAngle;
uniform float rainStrength;

const float noiseTextureResolution = 128.0;

float timeVal = sunAngle;

    float tSunrise  = ((clamp(timeVal, 0.96, 1.00)-0.96) / 0.04 + 1-(clamp(timeVal, 0.02, 0.15)-0.02) / 0.13);
    float tNoon     = ((clamp(timeVal, 0.02, 0.15)-0.02) / 0.13   - (clamp(timeVal, 0.35, 0.48)-0.35) / 0.13);
    float tSunset   = ((clamp(timeVal, 0.35, 0.48)-0.35) / 0.13   - (clamp(timeVal, 0.50, 0.53)-0.50) / 0.03);
    float tNight    = ((clamp(timeVal, 0.50, 0.53)-0.50) / 0.03   - (clamp(timeVal, 0.96, 1.00)-0.96) / 0.03);


float noise2De(in vec2 coord, in float size, in vec2 offset) {
    coord      *= size;
    coord      += offset;
    coord      /= noiseTextureResolution;
    return texture2D(noisetex, coord).r*2.0-1.0;
}


float waterH(vec3 wpos) {
    float windAnim = -frameTimeCounter*0.2;
    
    vec2 tick = vec2(windAnim, 0.0);
    vec3 wind = vec3(tick.x, 0.5*windAnim, tick.y);

    vec3 rpos = wpos*0.8;
    rpos.y *= 0.0;

    #if weakerWaterNormals == 0
    float noise = noise2De(rpos.xz+rpos.y, 0.06, tick);
        noise += noise2De(rpos.xz+rpos.y, 0.8, -tick*1.2)*0.025;
        noise += noise2De(rpos.xz+rpos.y, 1.4, tick*1.5)*0.0125;
        noise += noise2De(rpos.xz+rpos.y, 1.6, -tick*1.9)*0.001125;
    #endif

    #if weakerWaterNormals == 1
    float noise = noise2De(rpos.xz+rpos.y, 0.1, tick);
        noise += noise2De(rpos.xz+rpos.y, 0.8, -tick*1.2)*0.025;
        noise += noise2De(rpos.xz+rpos.y, 1.4, tick*1.5)*0.0125;
        noise += noise2De(rpos.xz+rpos.y, 1.6, -tick*1.9)*0.001125;
    #endif

    #if weakerWaterNormals == 2
    float noise = noise2De(rpos.xz+rpos.y, 0.6, tick);
        noise += noise2De(rpos.xz+rpos.y, 1.0, -tick*1.2)*0.025;
        noise += noise2De(rpos.xz+rpos.y, 1.4, tick*1.5)*0.0125;
        noise += noise2De(rpos.xz+rpos.y, 1.7, -tick*1.9)*0.001125;
    #endif
	
    return noise * 0.12;
}

void main() {

/*DRAWBUFFERS:04*/
    gl_FragData[0] = texture2D(tex, texcoord.st)*color;
    gl_FragData[1] = vec4(vec3(tbnMatrix), 1.0);
}